#include "libs.hh"
#include "classes.hh"
#include "headers.hh"
namespace FractalSpace
{
  void assign_density(Group& group, Fractal& fractal)
  {
    //--------------------------------------------------------------------------------------------------------------------------------
    // equivalength grid_length if total volume done at highest resolution
    //--------------------------------------------------------------------------------------------------------------------------------
    const  double scale=(double)(fractal.get_grid_length()*Misc::pow(2,fractal.get_level_max()));
    //--------------------------------------------------------------------------------------------------------------------------------
    //inverse width of a cell at current level, width at level=level_max is unity
    //--------------------------------------------------------------------------------------------------------------------------------
    const double d_inv=pow(2.0,group.get_level()-fractal.get_level_max());
    vector  <double> dens(8,0.0);
    double d_x,d_y,d_z;
    vector <double> pos(3);
    //--------------------------------------------------------------------------------------------------------------------------------
    // loop over all points in the group
    //--------------------------------------------------------------------------------------------------------------------------------
    //
      for(auto &p_point : group.list_points)
      {
	Point& point=*p_point;
	if(point.list_particles.empty())
	  continue;
	//--------------------------------------------------------------------------------------------------------------------------------
	// loop over all particles associated with a point. only points with real_pointer=0,1,3,4,9,10,12,13
	// can have particles associated.
	//--------------------------------------------------------------------------------------------------------------------------------
	dens.assign(8,0.0);
	for(auto &ppart : point.list_particles)
	  {
	    Particle& particle=*ppart;
	    if(particle.get_mass() == 0.0)
	      continue;
	    //--------------------------------------------------------------------------------------------------------------------------------
	    //find particle postion in cell
	    //--------------------------------------------------------------------------------------------------------------------------------
	    particle.get_pos(pos);
	    point.get_deltas(pos,d_x,d_y,d_z,scale,d_inv);
	    //--------------------------------------------------------------------------------------------------------------------------------
	    // add mass to the eight corners
	    //--------------------------------------------------------------------------------------------------------------------------------
	    Misc::add_dens<double>(dens,particle.get_mass(),d_x,d_y,d_z);
	  }
	//--------------------------------------------------------------------------------------------------------------------------------
	// add mass to the eight points forming the corners. These eight points will always all exist
	// because of the restriction of which points can have particles
	//--------------------------------------------------------------------------------------------------------------------------------
	point.add_density_at_points<double>(dens);
	point.set_mass_points(true);
      }
    //--------------------------------------------------------------------------------------------------------------------------------
    // scale from mass at point to density at point
    //--------------------------------------------------------------------------------------------------------------------------------
    if(group.get_set_scaling())
      {
	//--------------------------------------------------------------------------------------------------------------------------------
	// scaling is the inverse volume of a cell
	//--------------------------------------------------------------------------------------------------------------------------------
	double scaling=(double)(Misc::pow(fractal.get_grid_length(),3))*pow(8.0,group.get_level());
	for(auto &ppoint : group.list_points)
	  //--------------------------------------------------------------------------------------------------------------------------------
	  // use scaling from mass per point to real density
	  //--------------------------------------------------------------------------------------------------------------------------------
	  ppoint->scale_density_point(scaling);
      }
    double d_0=fractal.get_density_0();
    //--------------------------------------------------------------------------------------------------------------------------------
    // subtract mean density
    //--------------------------------------------------------------------------------------------------------------------------------
    if(group.get_set_dens() && d_0 != 0.0)
      group.subtract_density(d_0);
  }
}
