//----------------------------------------------------------------------------
//! \author $Author: jeffjohnson $
//! \date $Date: 2002-11-05 23:24:26 -0800 (Tue, 05 Nov 2002) $
//! \version $Revision: 461 $
//
// The LaplacianMatrixFactory class generates matrices representing the 
// laplacian operator.
//
// This class produces matrix entries for insertion into the linear solvers 
// supported by the Linear Solver Core (LSC) framework.  The matrix entries 
// are given in the double** structures that are returned by this factory, 
// and (needless to say) should be deallocated when one is finished with 
// them.
//----------------------------------------------------------------------------

#ifndef LAPLACIANMATRIXFACTORY_HH
#define LAPLACIANMATRIXFACTORY_HH

namespace Spheral {

template <typename Dimension>
class LaplacianMatrixFactory
{
  public:

  //! Here's the "grad squared" SPH Laplacian.
  static double** SPHGradSquared(const DataBase<Dimension>& db);

  private:

  // None of that autogenerated nonsense.
  LaplacianMatrixFactory();
  LaplacianMatrixFactory(const MatrixFactory&);
  LaplacianMatrixFactory& operator=(const MatrixFactory&);
  ~LaplacianMatrixFactory();

}; // end class MatrixFactory

} // end namespace Spheral

#endif
